@@ -1,4 +1,4 @@
-/*	$Id: document.c,v 1.176 2021/02/17 13:42:23 kristaps Exp $ */
+/*	$Id$ */
 /*
  * Copyright (c) 2008, Natacha Porté
  * Copyright (c) 2011, Vicent Martí
@@ -37,7 +37,7 @@
 /*
  * Make sure these are larger than enum hlist_fl.
  */
-#define HLIST_LI_END	(1 << 4) /* End of list item. */
+#define HLIST_LI_END	(1 << 6) /* End of list item. */
 
 /*
  * Mask of all list item types.
@@ -46,18 +46,6 @@
 			 HLIST_FL_ORDERED | \
 			 HLIST_FL_UNORDERED)
 
-/*
- * Used to hold metadata keys and values.
- * This is for filling in the metadata value with references.
- */
-struct	hbufn {
-	const struct lowdown_buf	*key; /* key of the value */
-	const struct lowdown_buf	*val; /* value (or NULL) */
-	TAILQ_ENTRY(hbufn) entries;
-};
-
-TAILQ_HEAD(hbufq, hbufn);
-
 /* 
  * Reference to a link.
  */
@@ -94,7 +82,7 @@ struct 	lowdown_doc {
 	int			  in_link_body; /* parsing link body */
 	size_t			  nodes; /* number of nodes */
 	struct lowdown_node	 *current; /* current node */
-	struct hbufq		  metaq; /* raw metadata key/values */
+	struct lowdown_metaq	 *metaq; /* raw metadata key/values */
 	size_t			  depth; /* current parse tree depth */
 	size_t			  maxdepth; /* max parse tree depth */
 	char			**meta; /* primer metadata */
@@ -1235,8 +1223,6 @@ char_langle_tag(struct lowdown_doc *doc,
 			n->rndr_autolink.type = altype;
 			if (!pushlbuf(&n->rndr_autolink.link, u_link))
 				goto err;
-			if (!pushlbuf(&n->rndr_autolink.text, u_link))
-				goto err;
 			popnode(doc, n);
 		} else {
 			n = pushnode(doc, LOWDOWN_RAW_HTML);
@@ -1442,7 +1428,7 @@ char_link(struct lowdown_doc *doc,
 	struct link_ref 	*lr;
 	struct foot_ref	 	*fr;
 	struct lowdown_node 	*n;
-	struct hbufn		*m;
+	struct lowdown_meta	*m;
 
 	is_img = offset && data[-1] == '!' && 
 		!is_escaped(data - offset, offset - 1);
@@ -1461,6 +1447,20 @@ char_link(struct lowdown_doc *doc,
 	else 
 		goto cleanup;
 
+	/*
+	 * If we start as an image then change into metadata or a
+	 * footnote, make sure to emit the exclamation mark.
+	 */
+
+	if (is_img && (is_footnote || is_metadata)) {
+		n = pushnode(doc, LOWDOWN_NORMAL_TEXT);
+		if (n == NULL)
+			goto err;
+		if (!pushbuf(&n->rndr_normal_text.text, &data[-1], 1))
+			goto err;
+		popnode(doc, n);
+	}
+
 	/* 
 	 * Footnote (in footer): look up footnote by its key in our
 	 * array of footnotes.  If we've already listed the footnote,
@@ -1536,18 +1536,17 @@ char_link(struct lowdown_doc *doc,
 
 		/* FIXME: slow O(n). */
 
-		TAILQ_FOREACH(m, &doc->metaq, entries) {
-			if (!hbuf_eq(m->key, &id))
+		TAILQ_FOREACH(m, doc->metaq, entries) {
+			if (!hbuf_streq(&id, m->key))
 				continue;
-			if (m->val != NULL) {
-				n = pushnode(doc, LOWDOWN_NORMAL_TEXT);
-				if (n == NULL)
-					goto err;
-				if (!pushlbuf
-				    (&n->rndr_normal_text.text, m->val))
-					goto err;
-				popnode(doc, n);
-			}
+			assert(m->value != NULL);
+			n = pushnode(doc, LOWDOWN_NORMAL_TEXT);
+			if (n == NULL)
+				goto err;
+			if (!pushbuf(&n->rndr_normal_text.text,
+			    m->value, strlen(m->value)))
+				goto err;
+			popnode(doc, n);
 			break;
 		}
 
@@ -2214,24 +2213,44 @@ prefix_oli(const struct lowdown_doc *doc,
 }
 
 /* 
- * Returns unordered list item prefix.
+ * Returns unordered list item prefix, including a GFM checkbox.  The
+ * "checked" pointer, if not NULL, is set to whether the check is set
+ * (>0), unset (=0), or not there (<0).
  */
 static size_t
-prefix_uli(const char *data, size_t size)
+prefix_uli(const struct lowdown_doc *doc,
+	const char *data, size_t size, int *checked)
 {
 	size_t	 i;
 
+	if (checked != NULL)
+		*checked = -1;
+
 	i = countspaces(data, 0, size, 3);
 
 	if (i + 1 >= size ||
 	    (data[i] != '*' && data[i] != '+' && 
 	     data[i] != '-') ||
-		data[i + 1] != ' ')
+	    data[i + 1] != ' ')
 		return 0;
 
 	if (is_next_headerline(data + i, size - i))
 		return 0;
 
+	if (!(doc->ext_flags & LOWDOWN_TASKLIST) || i + 5 >= size)
+		return i + 2;
+
+	if (data[i + 2] == '[' &&
+	    (data[i + 3] == ' ' ||
+	     data[i + 3] == 'x' ||
+	     data[i + 3] == 'X') &&
+	    data[i + 4] == ']' &&
+	    data[i + 5] == ' ') {
+		if (checked != NULL)
+			*checked = data[i + 3] != ' ';
+		return i + 6;
+	}
+
 	return i + 2;
 }
 
@@ -2534,14 +2553,14 @@ parse_listitem(struct lowdown_buf *ob, struct lowdown_doc *doc,
 				 orgpre, i, has_next_uli = 0, dli_lines,
 				 has_next_oli = 0, has_next_dli = 0;
 	int			 in_empty = 0, has_inside_empty = 0,
-				 in_fence = 0, ff;
+				 in_fence = 0, ff, checked = -1;
 	struct lowdown_node	*n;
 
 	/* Keeping track of the first indentation prefix. */
 
 	orgpre = countspaces(data, 0, size, 3);
 
-	beg = prefix_uli(data, size);
+	beg = prefix_uli(doc, data, size, &checked);
 
 	if (!beg)
 		beg = prefix_oli(doc, data, size, NULL);
@@ -2609,8 +2628,8 @@ parse_listitem(struct lowdown_buf *ob, struct lowdown_doc *doc,
 		 */
 
 		if (!in_fence) {
-			has_next_uli = prefix_uli
-				(data + beg + i, end - beg - i);
+			has_next_uli = prefix_uli(doc,
+				data + beg + i, end - beg - i, NULL);
 			has_next_dli =  dli_lines <= 2 && prefix_dli
 				(doc, data + beg + i, end - beg - i);
 			has_next_oli = prefix_oli
@@ -2696,6 +2715,11 @@ parse_listitem(struct lowdown_buf *ob, struct lowdown_doc *doc,
 	n->rndr_listitem.flags = *flags;
 	n->rndr_listitem.num = num;
 
+	if (checked > 0)
+		n->rndr_listitem.flags |= HLIST_FL_CHECKED;
+	else if (checked == 0)
+		n->rndr_listitem.flags |= HLIST_FL_UNCHECKED;
+
 	if (*flags & HLIST_FL_BLOCK) {
 		/* Intermediate render of block li. */
 
@@ -2817,8 +2841,7 @@ parse_list(struct lowdown_doc *doc,
 	char *data, size_t size, const char *oli_data)
 {
 	struct lowdown_buf	*work = NULL;
-	const char	    	*er = NULL;
-	size_t	 	     	 i = 0, k = 1;
+	size_t	 	     	 i = 0, pos;
 	ssize_t			 ret;
 	enum hlist_fl	     	 flags;
 	struct lowdown_node 	*n;
@@ -2829,20 +2852,20 @@ parse_list(struct lowdown_doc *doc,
 		goto err;
 	if ((n = pushnode(doc, LOWDOWN_LIST)) == NULL)
 		goto err;
+	n->rndr_list.start = 1;
 	n->rndr_list.flags = flags;
 
-	/* Set start point appropriately. */
-
 	if (oli_data != NULL && oli_data[0] != '\0') {
-		memcpy(n->rndr_list.start, oli_data,
-			sizeof(n->rndr_list.start));
-		k = strtonum(oli_data, 0, UINT32_MAX, &er);
-		assert(er == NULL);
+		n->rndr_list.start = strtonum
+			(oli_data, 0, UINT32_MAX, NULL);
+		if (n->rndr_list.start == 0)
+			n->rndr_list.start = 1;
 	}
 
+	pos = n->rndr_list.start;
 	while (i < size) {
 		ret = parse_listitem(work, doc, 
-			data + i, size - i, &flags, k++);
+			data + i, size - i, &flags, pos++);
 		if (ret < 0)
 			goto err;
 		i += ret;
@@ -3596,7 +3619,7 @@ parse_block(struct lowdown_doc *doc, char *data, size_t size)
 
 		/* Some sort of unordered list. */
 
-		if (prefix_uli(txt_data, end)) {
+		if (prefix_uli(doc, txt_data, end, NULL)) {
 			rc = parse_list(doc, txt_data, end, NULL);
 			if (rc < 0)
 				return 0;
@@ -4141,7 +4164,7 @@ static int
 parse_metadata(struct lowdown_doc *doc, const char *data, size_t sz)
 {
 	size_t	 	 	 i, j, pos = 0, vsz, keysz;
-	struct hbufn		*m;
+	struct lowdown_meta	*m;
 	struct lowdown_node	*n, *nn;
 	const char		*val, *key;
 	char			*cp, *buf;
@@ -4164,8 +4187,8 @@ parse_metadata(struct lowdown_doc *doc, const char *data, size_t sz)
 		return 0;
 
 	/*
-	 * Also put the metadata into the document's metaq because we
-	 * might set variables.
+	 * Put the metadata into the document's metaq because we might
+	 * set variables.
 	 */
 
 	for (pos = 0; pos < sz; ) {
@@ -4200,9 +4223,11 @@ parse_metadata(struct lowdown_doc *doc, const char *data, size_t sz)
 		 * both the local queue and the meta nodes.
 		 */
 
-		TAILQ_FOREACH(m, &doc->metaq, entries)
-			if (hbuf_streq(m->key, buf)) {
-				TAILQ_REMOVE(&doc->metaq, m, entries);
+		TAILQ_FOREACH(m, doc->metaq, entries)
+			if (strcmp(m->key, buf) == 0) {
+				TAILQ_REMOVE(doc->metaq, m, entries);
+				free(m->key);
+				free(m->value);
 				free(m);
 				break;
 			}
@@ -4227,33 +4252,50 @@ parse_metadata(struct lowdown_doc *doc, const char *data, size_t sz)
 		}
 		free(buf);
 
-		if ((m = calloc(1, sizeof(struct hbufn))) == NULL)
+		m = calloc(1, sizeof(struct lowdown_meta));
+		if (m == NULL)
+			return -1;
+		TAILQ_INSERT_TAIL(doc->metaq, m, entries);
+
+		m->key = strndup
+			(n->rndr_meta.key.data,
+			 n->rndr_meta.key.size);
+		if (m->key == NULL)
 			return -1;
-		TAILQ_INSERT_TAIL(&doc->metaq, m, entries);
-		m->key = &n->rndr_meta.key;
 
 		if (i == sz) {
+			if ((m->value = strdup("")) == NULL)
+				return -1;
 			popnode(doc, n);
 			break;
 		}
 
-		/* Parse the value, creating a node if nonempty. */
+		/*
+		 * Parse the value, creating a node if nonempty.  Make
+		 * sure that the metadata has an empty value if there's
+		 * no value to be parsed.
+		 */
 
 		assert(data[i] == ':');
 		i++;
 		while (i < sz && isspace((unsigned char)data[i]))
 			i++;
 		if (i == sz) {
+			if ((m->value = strdup("")) == NULL)
+				return -1;
 			popnode(doc, n);
 			break;
 		}
 
 		val = parse_metadata_val(&data[i], sz - i, &vsz);
+
+		if ((m->value = strndup(val, vsz)) == NULL)
+			return -1;
 		if ((nn = pushnode(doc, LOWDOWN_NORMAL_TEXT)) == NULL)
 			return -1;
-		m->val = &nn->rndr_normal_text.text;
 		if (!pushbuf(&nn->rndr_normal_text.text, val, vsz))
 			return -1;
+
 		popnode(doc, nn);
 		popnode(doc, n);
 
@@ -4270,17 +4312,27 @@ parse_metadata(struct lowdown_doc *doc, const char *data, size_t sz)
  * (Obviously only applicable if LOWDOWN_METADATA has been set.)
  */
 struct lowdown_node *
-lowdown_doc_parse(struct lowdown_doc *doc,
-	size_t *maxn, const char *data, size_t size)
+lowdown_doc_parse(struct lowdown_doc *doc, size_t *maxn,
+	const char *data, size_t size, struct lowdown_metaq *metaq)
 {
 	static const char 	 UTF8_BOM[] = { 0xEF, 0xBB, 0xBF };
 	struct lowdown_buf	*text;
 	size_t		 	 beg, end, i;
 	const char		*sv;
 	struct lowdown_node 	*n, *root = NULL;
-	struct hbufn		*m;
+	struct lowdown_metaq	 mq;
 	int			 c, rc = 0;
 
+	/*
+	 * Have a temporary "mq" if "metaq" is not set.  We clear this
+	 * automatically at the tail of the function.
+	 */
+
+	TAILQ_INIT(&mq);
+
+	if (metaq == NULL)
+		metaq = &mq;
+
 	/* Initialise the parser. */
 
 	doc->nodes = 0;
@@ -4288,7 +4340,9 @@ lowdown_doc_parse(struct lowdown_doc *doc,
 	doc->current = NULL;
 	doc->in_link_body = 0;
 	doc->foots = 0;
-	TAILQ_INIT(&doc->metaq);
+	doc->metaq = metaq;
+
+	TAILQ_INIT(doc->metaq);
 	TAILQ_INIT(&doc->refq);
 	TAILQ_INIT(&doc->footq);
 
@@ -4424,13 +4478,7 @@ out:
 	hbuf_free(text);
 	free_link_refs(&doc->refq);
 	free_foot_refq(&doc->footq);
-
-	/* The contents of these hbufs have been copied elsewhere. */
-
-	while ((m = TAILQ_FIRST(&doc->metaq)) != NULL) {
-		TAILQ_REMOVE(&doc->metaq, m, entries);
-		free(m);
-	}
+	lowdown_metaq_free(&mq);
 
 	if (rc) {
 		if (maxn != NULL)
@@ -4466,7 +4514,6 @@ lowdown_node_free(struct lowdown_node *p)
 		hbuf_free(&p->rndr_entity.text);
 		break;
 	case LOWDOWN_LINK_AUTO:
-		hbuf_free(&p->rndr_autolink.text);
 		hbuf_free(&p->rndr_autolink.link);
 		break;
 	case LOWDOWN_RAW_HTML:
