@@ -1,4 +1,4 @@
-/*	$Id: gemini.c,v 1.11 2021/02/18 22:37:50 kristaps Exp $ */
+/*	$Id$ */
 /*
  * Copyright (c) 2020--2021 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -59,10 +59,12 @@ struct gemini {
 	unsigned int		 flags; /* output flags */
 	ssize_t			 last_blank; /* line breaks or -1 (start) */
 	struct lowdown_buf	*tmp; /* for temporary allocations */
+	size_t			 nolinkqsz; /* if >0, don't record links */
 	struct linkq		 linkq; /* link queue */
 	size_t			 linkqsz; /* position in link queue */
 	wchar_t			*buf; /* buffer for counting wchar */
 	size_t			 bufsz; /* size of buf */
+	ssize_t			 headers_offs; /* header offset */
 };
 
 /*
@@ -85,10 +87,10 @@ link_freeq(struct linkq *q)
 
 static int
 rndr_link_ref(const struct gemini *st,
-	struct lowdown_buf *out, const struct link *l, int nl)
+	struct lowdown_buf *out, size_t ref, int nl)
 {
 	char		 buf[32], c;
-	size_t		 ref = l->id, sz = 0, i;
+	size_t		 sz = 0, i;
 
 	assert(ref);
 
@@ -310,6 +312,8 @@ rndr_meta(struct gemini *st,
 	struct lowdown_buf		*tmp = NULL;
 	struct lowdown_meta		*m;
 	const struct lowdown_node	*child;
+	ssize_t				 val;
+	const char			*ep;
 
 	/*
 	 * Manually render the children of the meta into a
@@ -340,6 +344,18 @@ rndr_meta(struct gemini *st,
 	if (m->value == NULL)
 		goto err;
 
+	if (strcmp(m->key, "shiftheadinglevelby") == 0) {
+		val = (ssize_t)strtonum
+			(m->value, -100, 100, &ep);
+		if (ep == NULL)
+			st->headers_offs = val + 1;
+	} else if (strcmp(m->key, "baseheaderlevel") == 0) {
+		val = (ssize_t)strtonum
+			(m->value, 1, 100, &ep);
+		if (ep == NULL)
+			st->headers_offs = val;
+	}
+
 	hbuf_free(tmp);
 	st->last_blank = last_blank;
 	return 1;
@@ -357,6 +373,8 @@ rndr_flush_linkq(struct gemini *st, struct lowdown_buf *out)
 	struct link	*l;
 	int		 rc;
 
+	assert(st->nolinkqsz == 0);
+
 	while ((l = TAILQ_FIRST(&st->linkq)) != NULL) {
 		TAILQ_REMOVE(&st->linkq, l, entries);
 		if (!HBUF_PUTSL(out, "=> "))
@@ -371,11 +389,13 @@ rndr_flush_linkq(struct gemini *st, struct lowdown_buf *out)
 			rc = 1;
 		if (!rc)
 			return 0;
-		if (!rndr_link_ref(st, out, l, 1))
+		if (!rndr_link_ref(st, out, l->id, 1))
 			return 0;
 		st->last_blank = 1;
 		free(l);
 	}
+
+	st->linkqsz = 0;
 	return 1;
 }
 
@@ -448,10 +468,11 @@ rndr_table(struct lowdown_buf *ob, struct gemini *st,
 
 	/*
 	 * Begin by counting the number of printable columns in each
-	 * column in each row.
+	 * column in each row.  Don't let us accumulate any links, as
+	 * we're going to re-run this after.
 	 */
 
-	assert(st->linkqsz == 0);
+	st->nolinkqsz = st->linkqsz + 1;
 	TAILQ_FOREACH(top, &n->children, entries) {
 		assert(top->type == LOWDOWN_TABLE_HEADER ||
 			top->type == LOWDOWN_TABLE_BODY);
@@ -472,15 +493,7 @@ rndr_table(struct lowdown_buf *ob, struct gemini *st,
 				st->last_blank = last_blank;
 			}
 	}
-
-	/* 
-	 * We may have accumulated links when temporarily rendering.
-	 * Remove these now, as we're going to have them printed for
-	 * real in the subsequent block.
-	 */
-
-	link_freeq(&st->linkq);
-	st->linkqsz = 0;
+	st->nolinkqsz = 0;
 
 	/* Now actually print, row-by-row into the output. */
 
@@ -590,9 +603,10 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 	struct gemini *st, const struct lowdown_node *n)
 {
 	const struct lowdown_node	*child, *prev;
-	int32_t				 entity;
-	size_t				 i;
 	struct link			*l;
+	size_t				 i;
+	ssize_t				 level;
+	int32_t				 entity;
 	int				 rc = 1;
 	
 	prev = n->parent == NULL ? NULL :
@@ -689,7 +703,11 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 		st->last_blank = -1;
 		break;
 	case LOWDOWN_HEADER:
-		for (i = 0; i <= n->rndr_header.level; i++)
+		level = (ssize_t)n->rndr_header.level +
+			st->headers_offs;
+		if (level < 1)
+			level = 1;
+		for (i = 0; i < (size_t)level; i++)
 			if (!HBUF_PUTSL(st->tmp, "#"))
 				return 0;
 		rc = HBUF_PUTSL(st->tmp, " ") &&
@@ -727,11 +745,17 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 		st->last_blank = -1;
 		break;
 	case LOWDOWN_LISTITEM:
-		if (n->rndr_listitem.flags & HLIST_FL_ORDERED)
+		if (n->rndr_listitem.flags & HLIST_FL_DEF)
+			rc = HBUF_PUTSL(st->tmp, ": ");
+		else if (n->rndr_listitem.flags & HLIST_FL_CHECKED)
+			rc = HBUF_PUTSL(st->tmp, "☑ ");
+		else if (n->rndr_listitem.flags & HLIST_FL_UNCHECKED)
+			rc = HBUF_PUTSL(st->tmp, "☐ ");
+		else if (n->rndr_listitem.flags & HLIST_FL_UNORDERED)
+			rc = HBUF_PUTSL(st->tmp, "* ");
+		else
 			rc = hbuf_printf(st->tmp, "%zu. ", 
 				n->rndr_listitem.num);
-		else
-			rc = HBUF_PUTSL(st->tmp, "* ");
 		if (!rc)
 			return 0;
 		rc = rndr_buf(st, ob, n, st->tmp);
@@ -813,12 +837,16 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 		if (IS_STANDALONE_LINK(n, prev) ||
 		    (st->flags & LOWDOWN_GEMINI_LINK_IN))
 			break;
-		if ((l = calloc(1, sizeof(struct link))) == NULL)
-			return 0;
-		l->n = n;
-		l->id = ++st->linkqsz;
-		TAILQ_INSERT_TAIL(&st->linkq, l, entries);
-		rc = rndr_link_ref(st, st->tmp, l, 0) &&
+		if (st->nolinkqsz == 0) {
+			if ((l = calloc(1, sizeof(struct link))) == NULL)
+				return 0;
+			l->n = n;
+			l->id = ++st->linkqsz;
+			TAILQ_INSERT_TAIL(&st->linkq, l, entries);
+			i = l->id;
+		} else
+			i = st->nolinkqsz++;
+		rc = rndr_link_ref(st, st->tmp, i, 0) &&
 			rndr_buf(st, ob, n, st->tmp);
 		break;
 	case LOWDOWN_NORMAL_TEXT:
@@ -907,7 +935,8 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 	if (!rc)
 		return 0;
 
-	if (st->last_blank > 1 && !TAILQ_EMPTY(&st->linkq) &&
+	if (st->nolinkqsz == 0 && st->last_blank > 1 && 
+	    !TAILQ_EMPTY(&st->linkq) && 
 	    !(st->flags & LOWDOWN_GEMINI_LINK_END)) {
 		if (!rndr_flush_linkq(st, ob))
 			return 0;
@@ -921,29 +950,23 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 
 int
 lowdown_gemini_rndr(struct lowdown_buf *ob,
-	struct lowdown_metaq *mq, void *arg, 
-	const struct lowdown_node *n)
+	void *arg, const struct lowdown_node *n)
 {
 	struct gemini		*st = arg;
 	int			 c;
 	struct lowdown_metaq	 metaq;
 
-	if (mq == NULL) {
-		TAILQ_INIT(&metaq);
-		mq = &metaq;
-	}
-
-	/* Set ourselves into a sane state. */
-
+	TAILQ_INIT(&metaq);
 	st->last_blank = 0;
+	st->headers_offs = 1;
 
-	c = rndr(ob, mq, st, n);
+	c = rndr(ob, &metaq, st, n);
 
 	link_freeq(&st->linkq);
 	st->linkqsz = 0;
+	st->nolinkqsz = 0;
 
-	if (mq == &metaq)
-		lowdown_metaq_free(mq);
+	lowdown_metaq_free(&metaq);
 	return c;
 }
 
