@@ -1,4 +1,4 @@
-/*	$Id: html.c,v 1.102 2021/02/16 23:15:41 kristaps Exp $ */
+/*	$Id$ */
 /*
  * Copyright (c) 2008, Natacha Porté
  * Copyright (c) 2011, Vicent Martí
@@ -23,6 +23,7 @@
 # include <sys/queue.h>
 #endif
 
+#include <assert.h>
 #include <ctype.h>
 #include <inttypes.h>
 #include <stdint.h>
@@ -50,8 +51,9 @@ struct	hentry {
  */
 struct 	html {
 	TAILQ_HEAD(, hentry) 	 headers_used;
-	size_t			 base_header_level; /* header offset */
+	ssize_t			 headers_offs; /* header offset */
 	unsigned int 		 flags; /* "oflags" in lowdown_opts */
+	int			 noescape; /* don't escape text */
 };
 
 /*
@@ -63,6 +65,7 @@ escape_html(struct lowdown_buf *ob, const char *source,
 	size_t length, const struct html *st)
 {
 
+	assert(st->noescape == 0);
 	return hesc_html(ob, source, length, 
 		st->flags & LOWDOWN_HTML_OWASP, 0,
 		st->flags & LOWDOWN_HTML_NUM_ENT);
@@ -76,7 +79,9 @@ escape_htmlb(struct lowdown_buf *ob,
 	const struct lowdown_buf *in, const struct html *st)
 {
 
-	return escape_html(ob, in->data, in->size, st);
+	return st->noescape ?
+		hbuf_putb(ob, in) :
+		escape_html(ob, in->data, in->size, st);
 }
 
 /*
@@ -89,6 +94,7 @@ escape_literal(struct lowdown_buf *ob,
 	const struct lowdown_buf *in, const struct html *st)
 {
 
+	assert(st->noescape == 0);
 	return hesc_html(ob, in->data, in->size, 
 		st->flags & LOWDOWN_HTML_OWASP, 1,
 		st->flags & LOWDOWN_HTML_NUM_ENT);
@@ -99,9 +105,11 @@ escape_literal(struct lowdown_buf *ob,
  * Return zero on failure, non-zero on success.
  */
 static int
-escape_href(struct lowdown_buf *ob, const struct lowdown_buf *in)
+escape_href(struct lowdown_buf *ob, const struct lowdown_buf *in,
+	const struct html *st)
 {
 
+	assert(st->noescape == 0);
 	return hesc_href(ob, in->data, in->size);
 }
 
@@ -129,7 +137,7 @@ rndr_autolink(struct lowdown_buf *ob,
 		return 0;
 	if (parm->type == HALINK_EMAIL && !HBUF_PUTSL(ob, "mailto:"))
 		return 0;
-	if (!escape_href(ob, &parm->link))
+	if (!escape_href(ob, &parm->link, st))
 		return 0;
 	if (!HBUF_PUTSL(ob, "\">"))
 		return 0;
@@ -164,7 +172,7 @@ rndr_blockcode(struct lowdown_buf *ob,
 	if (parm->lang.size) {
 		if (!HBUF_PUTSL(ob, "<pre><code class=\"language-"))
 			return 0;
-		if (!escape_href(ob, &parm->lang))
+		if (!escape_href(ob, &parm->lang, st))
 			return 0;
 		if (!HBUF_PUTSL(ob, "\">"))
 			return 0;
@@ -330,7 +338,7 @@ rndr_header_id(struct lowdown_buf *ob,
 
 	/* Convert to escaped values. */
 
-	if (!escape_href(ob, header))
+	if (!escape_href(ob, header, st))
 		return 0;
 
 	/*
@@ -360,11 +368,12 @@ rndr_header(struct lowdown_buf *ob,
 	const struct rndr_header *param, 
 	struct html *st)
 {
-	size_t	level = param->level + st->base_header_level;
+	ssize_t	level;
 
-	/* HTML doesn't allow greater than <h6>. */
-
-	if (level > 6)
+	level = (ssize_t)param->level + st->headers_offs;
+	if (level < 1)
+		level = 1;
+	else if (level > 6)
 		level = 6;
 
 	if (ob->size && !hbuf_putc(ob, '\n'))
@@ -396,7 +405,7 @@ rndr_link(struct lowdown_buf *ob,
 
 	if (!HBUF_PUTSL(ob, "<a href=\""))
 		return 0;
-	if (!escape_href(ob, &param->link))
+	if (!escape_href(ob, &param->link, st))
 		return 0;
 	if (param->title.size) {
 		if (!HBUF_PUTSL(ob, "\" title=\""))
@@ -420,9 +429,9 @@ rndr_list(struct lowdown_buf *ob,
 	if (ob->size && !hbuf_putc(ob, '\n'))
 		return 0;
 	if (param->flags & HLIST_FL_ORDERED) {
-		if (param->start[0] != '\0') {
+		if (param->start > 1) {
 			if (!hbuf_printf(ob, 
-			    "<ol start=\"%s\">\n", param->start))
+			    "<ol start=\"%zu\">\n", param->start))
 				return 0;
 		} else {
 			if (!HBUF_PUTSL(ob, "<ol>\n"))
@@ -484,6 +493,15 @@ rndr_listitem(struct lowdown_buf *ob,
 	if (blk && !HBUF_PUTSL(ob, "<p>"))
 		return 0;
 
+	if (n->rndr_listitem.flags &
+	    (HLIST_FL_CHECKED|HLIST_FL_UNCHECKED))
+		HBUF_PUTSL(ob, "<input type=\"checkbox\" ");
+	if (n->rndr_listitem.flags & HLIST_FL_CHECKED)
+		HBUF_PUTSL(ob, "checked=\"checked\" ");
+	if (n->rndr_listitem.flags &
+	    (HLIST_FL_CHECKED|HLIST_FL_UNCHECKED))
+		HBUF_PUTSL(ob, "/>");
+
 	/* Cut off any trailing space. */
 
 	if ((size = content->size) > 0) {
@@ -635,7 +653,7 @@ rndr_image(struct lowdown_buf *ob,
 
 	if (!HBUF_PUTSL(ob, "<img src=\""))
 		return 0;
-	if (!escape_href(ob, &param->link))
+	if (!escape_href(ob, &param->link, st))
 		return 0;
 	if (!HBUF_PUTSL(ob, "\" alt=\""))
 		return 0;
@@ -857,7 +875,7 @@ rndr_footnote_def(struct lowdown_buf *ob,
 		break;
 	}
 
-	if (!hbuf_printf(ob, "\n<li id=\"fn%zu\">\n", param->num + 1))
+	if (!hbuf_printf(ob, "\n<li id=\"fn%zu\">\n", param->num))
 		return 0;
 
 	if (pfound) {
@@ -866,7 +884,7 @@ rndr_footnote_def(struct lowdown_buf *ob,
 		if (!hbuf_printf(ob, "&#160;"
 		    "<a href=\"#fnref%zu\" rev=\"footnote\">"
 		    "&#8617;"
-		    "</a>", param->num + 1))
+		    "</a>", param->num))
 			return 0;
 		if (!hbuf_put(ob, 
 		    content->data + i, content->size - i))
@@ -888,7 +906,7 @@ rndr_footnote_ref(struct lowdown_buf *ob,
 		"<sup id=\"fnref%zu\">"
 		"<a href=\"#fn%zu\" rel=\"footnote\">"
 		"%zu</a></sup>", 
-		param->num + 1, param->num + 1, param->num + 1);
+		param->num, param->num, param->num);
 }
 
 static int
@@ -941,12 +959,15 @@ rndr_root(struct lowdown_buf *ob,
  * Return zero on failure, non-zero on success.
  */
 static int
-rndr_meta_multi(struct lowdown_buf *ob, const char *b,
+rndr_meta_multi(struct lowdown_buf *ob, const char *b, int href,
 	const char *starttag, const char *endtag)
 {
 	const char	*start;
 	size_t		 sz, i, bsz;
 
+	if (b == NULL)
+		return 1;
+
 	bsz = strlen(b);
 
 	for (i = 0; i < bsz; i++) {
@@ -970,7 +991,9 @@ rndr_meta_multi(struct lowdown_buf *ob, const char *b,
 			return 0;
 		if (!HBUF_PUTSL(ob, "\""))
 			return 0;
-		if (!hbuf_put(ob, start, sz))
+		if (!href && !hesc_attr(ob, start, sz))
+			return 0;
+		else if (href && !hesc_href(ob, start, sz))
 			return 0;
 		if (!HBUF_PUTSL(ob, "\""))
 			return 0;
@@ -994,6 +1017,8 @@ rndr_meta(struct lowdown_buf *ob,
 	const struct lowdown_node *n, struct html *st)
 {
 	struct lowdown_meta	*m;
+	ssize_t			 val;
+	const char		*ep;
 
 	m = calloc(1, sizeof(struct lowdown_meta));
 	if (m == NULL)
@@ -1008,11 +1033,16 @@ rndr_meta(struct lowdown_buf *ob,
 	if (m->value == NULL)
 		return 0;
 
-	if (strcasecmp(m->key, "baseheaderlevel") == 0) {
-		st->base_header_level = strtonum
-			(m->value, 1, 1000, NULL);
-		if (st->base_header_level == 0)
-			st->base_header_level = 1;
+	if (strcmp(m->key, "shiftheadinglevelby") == 0) {
+		val = (ssize_t)strtonum
+			(m->value, -100, 100, &ep);
+		if (ep == NULL)
+			st->headers_offs = val + 1;
+	} else if (strcmp(m->key, "baseheaderlevel") == 0) {
+		val = (ssize_t)strtonum
+			(m->value, 1, 100, &ep);
+		if (ep == NULL)
+			st->headers_offs = val;
 	}
 
 	return 1;
@@ -1073,39 +1103,47 @@ rndr_doc_header(struct lowdown_buf *ob,
 	if (rcsauthor != NULL)
 		author = rcsauthor;
 
-	if (affil != NULL && !rndr_meta_multi
-	    (ob, affil, "<meta name=\"creator\" content=", " />"))
+	if (!rndr_meta_multi(ob, affil, 0,
+	    "<meta name=\"creator\" content=", " />"))
 		return 0;
 
-	if (author != NULL && !rndr_meta_multi
-	    (ob, author, "<meta name=\"author\" content=", " />"))
+	if (!rndr_meta_multi(ob, author, 0,
+	    "<meta name=\"author\" content=", " />"))
 		return 0;
 
-	if (copy != NULL && !rndr_meta_multi
-	    (ob, copy, "<meta name=\"copyright\" content=", " />"))
+	if (!rndr_meta_multi(ob, copy, 0,
+	    "<meta name=\"copyright\" content=", " />"))
 		return 0;
 
-	if (css != NULL && !rndr_meta_multi
-	    (ob, css, "<link rel=\"stylesheet\" href=", " />"))
-		return 0;
+	/*
+	 * FIXME: don't use "scheme" if the date isn't in the
+	 * appropriate format, or modify it depending upon the position
+	 * of the year?
+	 */
 
 	if (date != NULL) {
 		if (!hbuf_printf(ob, "<meta name="
 		    "\"date\" scheme=\"YYYY-MM-DD\" content=\""))
 			return 0;
-		if (!hbuf_puts(ob, date))
+		if (!hesc_attr(ob, date, strlen(date)))
 			return 0;
 		if (!HBUF_PUTSL(ob, "\" />\n"))
 			return 0;
 	}
 
-	if (script != NULL && !rndr_meta_multi
-	    (ob, script, "<script src=", "></script>"))
+	if (!rndr_meta_multi(ob, css, 1,
+	    "<link rel=\"stylesheet\" href=", " />"))
+		return 0;
+
+	if (!rndr_meta_multi(ob, script, 1,
+	     "<script src=", "></script>"))
 		return 0;
 
 	if (!HBUF_PUTSL(ob, "<title>"))
 		return 0;
-	if (!hbuf_puts(ob, title))
+	if (!hesc_html(ob, title, strlen(title), 
+	    st->flags & LOWDOWN_HTML_OWASP, 0,
+	    st->flags & LOWDOWN_HTML_NUM_ENT))
 		return 0;
 	if (!HBUF_PUTSL(ob, "</title>\n"))
 		return 0;
@@ -1126,6 +1164,16 @@ rndr(struct lowdown_buf *ob,
 	if ((tmp = hbuf_new(64)) == NULL)
 		return 0;
 
+	/*
+	 * If we're processing metadata, don't escape the content as we
+	 * read and parse it.  This prevents double-escaping.  We'll
+	 * properly escape things as we inline them (standalone mode) or
+	 * when we write body text.
+	 */
+
+	if (n->type == LOWDOWN_META)
+		st->noescape = 1;
+
 	TAILQ_FOREACH(child, &n->children, entries)
 		if (!rndr(tmp, mq, st, child))
 			goto out;
@@ -1160,6 +1208,7 @@ rndr(struct lowdown_buf *ob,
 		rc = rndr_doc_header(ob, tmp, mq, st);
 		break;
 	case LOWDOWN_META:
+		st->noescape = 0;
 		if (n->chng != LOWDOWN_CHNG_DELETE)
 			rc = rndr_meta(ob, tmp, mq, n, st);
 		break;
@@ -1290,28 +1339,18 @@ out:
 
 int
 lowdown_html_rndr(struct lowdown_buf *ob,
-	struct lowdown_metaq *mq, void *arg, 
-	const struct lowdown_node *n)
+	void *arg, const struct lowdown_node *n)
 {
 	struct html		*st = arg;
 	struct lowdown_metaq	 metaq;
 	int			 rc;
 
-	/* Temporary metaq if not provided. */
+	TAILQ_INIT(&metaq);
+	st->headers_offs = 1;
 
-	if (mq == NULL) {
-		TAILQ_INIT(&metaq);
-		mq = &metaq;
-	}
-
-	st->base_header_level = 1;
-	rc = rndr(ob, mq, st, n);
-
-	/* Release temporary metaq. */
-
-	if (mq == &metaq)
-		lowdown_metaq_free(mq);
+	rc = rndr(ob, &metaq, st, n);
 
+	lowdown_metaq_free(&metaq);
 	return rc;
 }
 
