@@ -1,4 +1,4 @@
-/*	$Id: library.c,v 1.73 2021/01/23 21:49:58 kristaps Exp $ */
+/*	$Id$ */
 /*
  * Copyright (c) 2017--2021 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -89,9 +89,12 @@ lowdown_buf(const struct lowdown_opts *opts,
 
 	if ((doc = lowdown_doc_new(opts)) == NULL)
 		goto err;
-	if ((n = lowdown_doc_parse(doc, &maxn, data, datasz)) == NULL)
+
+	n = lowdown_doc_parse(doc, &maxn, data, datasz, metaq);
+	if (n == NULL)
 		goto err;
 	assert(n->type == LOWDOWN_ROOT);
+
     	if (opts != NULL && (opts->oflags & LOWDOWN_SMARTY)) 
 		if (!smarty(n, maxn, t))
 			goto err;
@@ -103,7 +106,7 @@ lowdown_buf(const struct lowdown_opts *opts,
 	case LOWDOWN_GEMINI:
 		if ((rndr = lowdown_gemini_new(opts)) == NULL)
 			goto err;
-		c = lowdown_gemini_rndr(ob, metaq, rndr, n);
+		c = lowdown_gemini_rndr(ob, rndr, n);
 		lowdown_gemini_free(rndr);
 		if (!c)
 			goto err;
@@ -111,7 +114,7 @@ lowdown_buf(const struct lowdown_opts *opts,
 	case LOWDOWN_HTML:
 		if ((rndr = lowdown_html_new(opts)) == NULL)
 			goto err;
-		c = lowdown_html_rndr(ob, metaq, rndr, n);
+		c = lowdown_html_rndr(ob, rndr, n);
 		lowdown_html_free(rndr);
 		if (!c)
 			goto err;
@@ -119,7 +122,7 @@ lowdown_buf(const struct lowdown_opts *opts,
 	case LOWDOWN_LATEX:
 		if ((rndr = lowdown_latex_new(opts)) == NULL)
 			goto err;
-		c = lowdown_latex_rndr(ob, metaq, rndr, n);
+		c = lowdown_latex_rndr(ob, rndr, n);
 		lowdown_latex_free(rndr);
 		if (!c)
 			goto err;
@@ -128,7 +131,7 @@ lowdown_buf(const struct lowdown_opts *opts,
 	case LOWDOWN_NROFF:
 		if ((rndr = lowdown_nroff_new(opts)) == NULL)
 			goto err;
-		c = lowdown_nroff_rndr(ob, metaq, rndr, n);
+		c = lowdown_nroff_rndr(ob, rndr, n);
 		lowdown_nroff_free(rndr);
 		if (!c)
 			goto err;
@@ -136,7 +139,7 @@ lowdown_buf(const struct lowdown_opts *opts,
 	case LOWDOWN_TERM:
 		if ((rndr = lowdown_term_new(opts)) == NULL)
 			goto err;
-		c = lowdown_term_rndr(ob, metaq, rndr, n);
+		c = lowdown_term_rndr(ob, rndr, n);
 		lowdown_term_free(rndr);
 		if (!c)
 			goto err;
@@ -164,8 +167,7 @@ int
 lowdown_buf_diff(const struct lowdown_opts *opts,
 	const char *new, size_t newsz,
 	const char *old, size_t oldsz,
-	char **res, size_t *rsz,
-	struct lowdown_metaq *metaq)
+	char **res, size_t *rsz)
 {
 	struct lowdown_buf 	*ob = NULL;
 	void 		 	*rndr = NULL;
@@ -181,10 +183,10 @@ lowdown_buf_diff(const struct lowdown_opts *opts,
 	if ((doc = lowdown_doc_new(opts)) == NULL)
 		goto err;
 
-	nnew = lowdown_doc_parse(doc, NULL, new, newsz);
+	nnew = lowdown_doc_parse(doc, NULL, new, newsz, NULL);
 	if (nnew == NULL)
 		goto err;
-	nold = lowdown_doc_parse(doc, NULL, old, oldsz);
+	nold = lowdown_doc_parse(doc, NULL, old, oldsz, NULL);
 	if (nold == NULL)
 		goto err;
 
@@ -206,7 +208,7 @@ lowdown_buf_diff(const struct lowdown_opts *opts,
 	case LOWDOWN_GEMINI:
 		if ((rndr = lowdown_gemini_new(opts)) == NULL)
 			goto err;
-		c = lowdown_gemini_rndr(ob, metaq, rndr, ndiff);
+		c = lowdown_gemini_rndr(ob, rndr, ndiff);
 		lowdown_gemini_free(rndr);
 		if (!c)
 			goto err;
@@ -214,7 +216,7 @@ lowdown_buf_diff(const struct lowdown_opts *opts,
 	case LOWDOWN_HTML:
 		if ((rndr = lowdown_html_new(opts)) == NULL)
 			goto err;
-		c = lowdown_html_rndr(ob, metaq, rndr, ndiff);
+		c = lowdown_html_rndr(ob, rndr, ndiff);
 		lowdown_html_free(rndr);
 		if (!c)
 			goto err;
@@ -222,7 +224,7 @@ lowdown_buf_diff(const struct lowdown_opts *opts,
 	case LOWDOWN_LATEX:
 		if ((rndr = lowdown_latex_new(opts)) == NULL)
 			goto err;
-		c = lowdown_latex_rndr(ob, metaq, rndr, ndiff);
+		c = lowdown_latex_rndr(ob, rndr, ndiff);
 		lowdown_latex_free(rndr);
 		if (!c)
 			goto err;
@@ -231,7 +233,7 @@ lowdown_buf_diff(const struct lowdown_opts *opts,
 	case LOWDOWN_NROFF:
 		if ((rndr = lowdown_nroff_new(opts)) == NULL)
 			goto err;
-		c = lowdown_nroff_rndr(ob, metaq, rndr, ndiff);
+		c = lowdown_nroff_rndr(ob, rndr, ndiff);
 		lowdown_nroff_free(rndr);
 		if (!c)
 			goto err;
@@ -239,7 +241,7 @@ lowdown_buf_diff(const struct lowdown_opts *opts,
 	case LOWDOWN_TERM:
 		if ((rndr = lowdown_term_new(opts)) == NULL)
 			goto err;
-		c = lowdown_term_rndr(ob, metaq, rndr, ndiff);
+		c = lowdown_term_rndr(ob, rndr, ndiff);
 		lowdown_term_free(rndr);
 		if (!c)
 			goto err;
@@ -288,8 +290,7 @@ out:
 
 int
 lowdown_file_diff(const struct lowdown_opts *opts,
-	FILE *fnew, FILE *fold, char **res, size_t *rsz, 
-	struct lowdown_metaq *metaq)
+	FILE *fnew, FILE *fold, char **res, size_t *rsz)
 {
 	struct lowdown_buf	*bnew = NULL, *bold = NULL;
 	int	 		 rc = 0;
@@ -306,7 +307,7 @@ lowdown_file_diff(const struct lowdown_opts *opts,
 	if (!lowdown_buf_diff(opts, 
 	    bnew->data, bnew->size, 
 	    bold->data, bold->size, 
-	    res, rsz, metaq))
+	    res, rsz))
 		goto out;
 	rc = 1;
 out:
