@@ -1,4 +1,4 @@
-/*	$Id: lowdown.h,v 1.101 2021/02/08 20:10:54 kristaps Exp $ */
+/*	$Id$ */
 /*
  * Copyright (c) 2017--2021 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -124,7 +124,9 @@ enum	hlist_fl {
 	HLIST_FL_ORDERED = (1 << 0), /* <ol> list item */
 	HLIST_FL_BLOCK = (1 << 1), /* <li> containing block data */
 	HLIST_FL_UNORDERED = (1 << 2), /* <ul> list item */
-	HLIST_FL_DEF = (1 << 3) /* <dl> list item */
+	HLIST_FL_DEF = (1 << 3), /* <dl> list item */
+	HLIST_FL_CHECKED = (1 << 4), /* <li> with checked box */
+	HLIST_FL_UNCHECKED = (1 << 5), /* <li> with unchecked box */
 };
 
 /*
@@ -164,7 +166,6 @@ struct	rndr_entity {
 
 struct	rndr_autolink {
 	struct lowdown_buf link; /* link address */
-	struct lowdown_buf text; /* shown address */
 	enum halink_type type; /* type of link */
 };
 
@@ -191,26 +192,29 @@ struct	rndr_codespan {
 };
 
 struct	rndr_table{
-	size_t columns; /* number of columns */
+	size_t columns;
 };
 
 struct	rndr_table_header {
-	enum htbl_flags *flags; /* per-column flags */
-	size_t columns; /* number of columns */
+	enum htbl_flags *flags;
+	size_t columns;
 };
 
 struct	rndr_table_cell {
-	enum htbl_flags flags; /* flags for cell */
-	size_t col; /* column number */
-	size_t columns; /* number of columns */
+	enum htbl_flags flags;
+	size_t col;
+	size_t columns;
 };
 
 struct	rndr_footnote_def {
-	size_t num; /* footnote number */
+	size_t num;
+	struct lowdown_buf key;
 };
 
 struct	rndr_footnote_ref {
-	size_t num; /* footnote number */
+	size_t num;
+	struct lowdown_buf def;
+	struct lowdown_buf key;
 };
 
 struct	rndr_blockhtml {
@@ -218,23 +222,13 @@ struct	rndr_blockhtml {
 };
 
 struct	rndr_list {
-	/*
-	 * This should only be checked for bit-wise
-	 * HLIST_FL_ORDERED OR HLIST_FL_BLOCK.
-	 * There may be other private bits set.
-	 */
 	enum hlist_fl flags;
-	/*
-	 * This is string of size >0 iff
-	 * HLIST_FL_ORDERED and we're parsing
-	 * CommonMark, else it's an empty string.
-	 */
-	char start[10];
+	size_t start;
 };
 
 struct	rndr_listitem {
-	enum hlist_fl flags; /* all possible flags */
-	size_t num; /* index in ordered */
+	enum hlist_fl flags;
+	size_t num;
 };
 
 struct	rndr_header{
@@ -318,6 +312,7 @@ struct	lowdown_opts {
 #define	LOWDOWN_COMMONMARK	  0x8000
 #define	LOWDOWN_DEFLIST		  0x10000
 #define	LOWDOWN_IMG_EXT	 	  0x20000
+#define LOWDOWN_TASKLIST	  0x40000
 	unsigned int		  oflags;
 #define LOWDOWN_HTML_SKIP_HTML	  0x01 /* skip all HTML */
 #define LOWDOWN_HTML_ESCAPE	  0x02 /* escape HTML (if not skip) */
@@ -333,6 +328,7 @@ struct	lowdown_opts {
 #define LOWDOWN_STANDALONE	  0x200 /* emit complete document */
 #define LOWDOWN_TERM_SHORTLINK	  0x400 /* shorten URLs */
 #define LOWDOWN_TERM_NOLINK	  0x20000 /* don't show URLs */
+#define	LOWDOWN_TERM_NOCOLOUR	  0x800000 /* no colours */
 #define	LOWDOWN_HTML_OWASP	  0x800 /* use OWASP escaping */
 #define	LOWDOWN_HTML_NUM_ENT	  0x1000 /* use &#nn; if possible */
 #define LOWDOWN_LATEX_SKIP_HTML	  0x2000 /* skip all HTML */
@@ -340,6 +336,8 @@ struct	lowdown_opts {
 #define	LOWDOWN_GEMINI_LINK_END	  0x8000 /* links at end */
 #define	LOWDOWN_GEMINI_LINK_IN	  0x10000 /* links inline */
 #define LOWDOWN_GEMINI_METADATA	  0x100000 /* show metadata */
+#define	LOWDOWN_GEMINI_LINK_NOREF 0x200000 /* for !inline, no names */
+#define	LOWDOWN_GEMINI_LINK_ROMAN 0x400000 /* roman link names */
 	char			**meta;
 	size_t			  metasz;
 	char			**metaovr;
@@ -360,11 +358,11 @@ int	 lowdown_buf(const struct lowdown_opts *,
 		char **, size_t *, struct lowdown_metaq *);
 int	 lowdown_buf_diff(const struct lowdown_opts *, 
 		const char *, size_t, const char *, size_t,
-		char **, size_t *, struct lowdown_metaq *);
+		char **, size_t *);
 int	 lowdown_file(const struct lowdown_opts *, 
 		FILE *, char **, size_t *, struct lowdown_metaq *);
 int	 lowdown_file_diff(const struct lowdown_opts *, FILE *, 
-		FILE *, char **, size_t *, struct lowdown_metaq *);
+		FILE *, char **, size_t *);
 
 /* 
  * Low-level functions.
@@ -379,8 +377,8 @@ void	 lowdown_buf_free(struct lowdown_buf *);
 struct lowdown_doc
 	*lowdown_doc_new(const struct lowdown_opts *);
 struct lowdown_node
-	*lowdown_doc_parse(struct lowdown_doc *,
-		size_t *, const char *, size_t);
+	*lowdown_doc_parse(struct lowdown_doc *, size_t *,
+		const char *, size_t, struct lowdown_metaq *);
 struct lowdown_node
 	*lowdown_diff(const struct lowdown_node *,
 		const struct lowdown_node *, size_t *);
@@ -391,26 +389,22 @@ void 	 lowdown_node_free(struct lowdown_node *);
 
 void	 lowdown_html_free(void *);
 void	*lowdown_html_new(const struct lowdown_opts *);
-int 	 lowdown_html_rndr(struct lowdown_buf *,
-		struct lowdown_metaq *, void *, 
+int 	 lowdown_html_rndr(struct lowdown_buf *, void *, 
 		const struct lowdown_node *);
 
 void	 lowdown_gemini_free(void *);
 void	*lowdown_gemini_new(const struct lowdown_opts *);
-int 	 lowdown_gemini_rndr(struct lowdown_buf *,
-		struct lowdown_metaq *, void *, 
+int 	 lowdown_gemini_rndr(struct lowdown_buf *, void *, 
 		const struct lowdown_node *);
 
 void	 lowdown_term_free(void *);
 void	*lowdown_term_new(const struct lowdown_opts *);
-int 	 lowdown_term_rndr(struct lowdown_buf *,
-		struct lowdown_metaq *, void *, 
+int 	 lowdown_term_rndr(struct lowdown_buf *, void *, 
 		const struct lowdown_node *);
 
 void	 lowdown_nroff_free(void *);
 void	*lowdown_nroff_new(const struct lowdown_opts *);
-int 	 lowdown_nroff_rndr(struct lowdown_buf *, 
-		struct lowdown_metaq *, void *, 
+int 	 lowdown_nroff_rndr(struct lowdown_buf *, void *, 
 		const struct lowdown_node *);
 
 int 	 lowdown_tree_rndr(struct lowdown_buf *, 
@@ -418,8 +412,7 @@ int 	 lowdown_tree_rndr(struct lowdown_buf *,
 
 void	 lowdown_latex_free(void *);
 void	*lowdown_latex_new(const struct lowdown_opts *);
-int 	 lowdown_latex_rndr(struct lowdown_buf *, 
-		struct lowdown_metaq *, void *, 
+int 	 lowdown_latex_rndr(struct lowdown_buf *, void *, 
 		const struct lowdown_node *);
 
 __END_DECLS
