@@ -1,4 +1,4 @@
-/*	$Id: main.c,v 1.102 2021/02/18 22:37:50 kristaps Exp $ */
+/*	$Id$ */
 /*
  * Copyright (c) 2016, 2017, 2020 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -26,6 +26,7 @@
 #endif
 #include <sys/ioctl.h>
 
+#include <assert.h>
 #if HAVE_ERR
 # include <err.h>
 #endif
@@ -264,6 +265,8 @@ main(int argc, char *argv[])
 		{ "term-no-shortlinks",	no_argument, 	&roflag, LOWDOWN_TERM_SHORTLINK },
 		{ "term-nolinks",	no_argument, 	&aoflag, LOWDOWN_TERM_NOLINK },
 		{ "term-no-nolinks",	no_argument, 	&roflag, LOWDOWN_TERM_NOLINK },
+		{ "term-no-colour",	no_argument, 	&aoflag, LOWDOWN_TERM_NOCOLOUR },
+		{ "term-colour",	no_argument, 	&roflag, LOWDOWN_TERM_NOCOLOUR },
 		{ "out-smarty",		no_argument,	&aoflag, LOWDOWN_SMARTY },
 		{ "out-no-smarty",	no_argument,	&roflag, LOWDOWN_SMARTY },
 		{ "out-standalone",	no_argument,	&aoflag, LOWDOWN_STANDALONE },
@@ -286,8 +289,8 @@ main(int argc, char *argv[])
 		{ "parse-no-math",	no_argument,	&riflag, LOWDOWN_MATH },
 		{ "parse-codeindent",	no_argument,	&riflag, LOWDOWN_NOCODEIND },
 		{ "parse-no-codeindent",no_argument,	&aiflag, LOWDOWN_NOCODEIND },
-		{ "parse-intraemph",	no_argument,	&aiflag, LOWDOWN_NOINTEM },
-		{ "parse-no-intraemph",	no_argument,	&riflag, LOWDOWN_NOINTEM },
+		{ "parse-intraemph",	no_argument,	&riflag, LOWDOWN_NOINTEM },
+		{ "parse-no-intraemph",	no_argument,	&aiflag, LOWDOWN_NOINTEM },
 		{ "parse-metadata",	no_argument,	&aiflag, LOWDOWN_METADATA },
 		{ "parse-no-metadata",	no_argument,	&riflag, LOWDOWN_METADATA },
 		{ "parse-cmark",	no_argument,	&aiflag, LOWDOWN_COMMONMARK },
@@ -296,6 +299,8 @@ main(int argc, char *argv[])
 		{ "parse-no-deflists",	no_argument,	&riflag, LOWDOWN_DEFLIST },
 		{ "parse-img-ext",	no_argument,	&aiflag, LOWDOWN_IMG_EXT },
 		{ "parse-no-img-ext",	no_argument,	&riflag, LOWDOWN_IMG_EXT },
+		{ "parse-tasklists",	no_argument,	&aiflag, LOWDOWN_TASKLIST },
+		{ "parse-no-tasklists",	no_argument,	&riflag, LOWDOWN_TASKLIST },
 		{ "parse-maxdepth",	required_argument, NULL, 5 },
 		{ NULL,			0,	NULL,	0 }
 	};
@@ -311,16 +316,18 @@ main(int argc, char *argv[])
 
 	opts.maxdepth = 128;
 	opts.type = LOWDOWN_HTML;
-	opts.feat = LOWDOWN_FOOTNOTES |
+	opts.feat =
 		LOWDOWN_AUTOLINK |
-		LOWDOWN_TABLES |
-		LOWDOWN_SUPER |
-		LOWDOWN_STRIKE |
-		LOWDOWN_FENCED |
 		LOWDOWN_COMMONMARK |
 		LOWDOWN_DEFLIST |
+		LOWDOWN_FENCED |
+		LOWDOWN_FOOTNOTES |
 		LOWDOWN_IMG_EXT |
-		LOWDOWN_METADATA;
+		LOWDOWN_METADATA |
+		LOWDOWN_STRIKE |
+		LOWDOWN_SUPER |
+		LOWDOWN_TABLES |
+		LOWDOWN_TASKLIST;
 	opts.oflags = 
 		LOWDOWN_HTML_ESCAPE |
 		LOWDOWN_HTML_HEAD_IDS |
@@ -451,9 +458,6 @@ main(int argc, char *argv[])
 	 * Non-diff mode takes an optional single argument.
 	 */
 
-	if (diff && extract != NULL)
-		errx(1, "-X not applicable to diff mode");
-
 	if ((diff && (argc == 0 || argc > 2)) || (!diff && argc > 1))
 		goto usage;
 
@@ -490,9 +494,19 @@ main(int argc, char *argv[])
 	if (extract)
 		opts.feat |= LOWDOWN_METADATA;
 
+	/* 
+	 * Allow NO_COLOUR to dictate colours.
+	 * This only works for -Tterm output when not in diff mode.
+	 */
+
+	if (getenv("NO_COLOR") != NULL ||
+	    getenv("NO_COLOUR") != NULL)
+		opts.oflags |= LOWDOWN_TERM_NOCOLOUR;
+
 	if (diff) {
+		opts.oflags &= ~LOWDOWN_TERM_NOCOLOUR;
 		if (!lowdown_file_diff
-		    (&opts, fin, din, &ret, &retsz, &mq))
+		    (&opts, fin, din, &ret, &retsz))
 			errx(1, "%s: failed parse", fnin);
 	} else {
 		if (!lowdown_file(&opts, fin, &ret, &retsz, &mq))
@@ -500,6 +514,7 @@ main(int argc, char *argv[])
 	}
 
 	if (extract != NULL) {
+		assert(!diff);
 		TAILQ_FOREACH(m, &mq, entries) 
 			if (strcasecmp(m->key, extract) == 0)
 				break;
