@@ -1,4 +1,4 @@
-/*	$Id: term.c,v 1.47 2021/02/17 10:05:48 kristaps Exp $ */
+/*	$Id$ */
 /*
  * Copyright (c) 2020--2021 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -148,7 +148,7 @@ static const struct sty sty_chng_del =	{ 0, 0, 0, 0, 100,  0, 0 };
 
 static const struct sty sty_ddata_pfx =	{ 0, 0, 0, 0,   0, 93, 0 };
 static const struct sty sty_fdef_pfx =	{ 0, 0, 0, 0,   0, 92, 1 };
-static const struct sty sty_bkqt_pfx =	{ 0, 0, 0, 0,   0, 37, 0 };
+static const struct sty sty_bkqt_pfx =	{ 0, 0, 0, 0,   0, 93, 0 };
 static const struct sty sty_oli_pfx =	{ 0, 0, 0, 0,   0, 93, 0 };
 static const struct sty sty_uli_pfx =	{ 0, 0, 0, 0,   0, 93, 0 };
 
@@ -244,7 +244,8 @@ rndr_escape(struct term *term, struct lowdown_buf *out,
  * Return zero on failure (memory), non-zero on success.
  */
 static int
-rndr_buf_style(struct lowdown_buf *out, const struct sty *s)
+rndr_buf_style(const struct term *term,
+	struct lowdown_buf *out, const struct sty *s)
 {
 	int	has = 0;
 
@@ -276,13 +277,13 @@ rndr_buf_style(struct lowdown_buf *out, const struct sty *s)
 		if (!HBUF_PUTSL(out, "9"))
 			return 0;
 	}
-	if (s->bcolour) {
+	if (s->bcolour && !(term->opts & LOWDOWN_TERM_NOCOLOUR)) {
 		if (has++ && !HBUF_PUTSL(out, ";"))
 			return 0;
 		if (!hbuf_printf(out, "%zu", s->bcolour))
 			return 0;
 	}
-	if (s->colour) {
+	if (s->colour && !(term->opts & LOWDOWN_TERM_NOCOLOUR)) {
 		if (has++ && !HBUF_PUTSL(out, ";"))
 			return 0;
 		if (!hbuf_printf(out, "%zu", s->colour))
@@ -458,10 +459,11 @@ rndr_buf_startline_prefixes(struct term *term,
 	struct sty *s, const struct lowdown_node *n,
 	struct lowdown_buf *out)
 {
-	size_t	 			 i, emit;
-	int	 		 	 pstyle = 0;
 	struct sty			 sinner;
 	const struct lowdown_node	*np;
+	size_t	 			 i, emit;
+	int	 		 	 pstyle = 0;
+	enum hlist_fl			 fl;
 
 	if (n->parent != NULL &&
 	    !rndr_buf_startline_prefixes(term, s, n->parent, out))
@@ -501,6 +503,7 @@ rndr_buf_startline_prefixes(struct term *term,
 	switch (n->type) {
 	case LOWDOWN_TABLE_BLOCK:
 	case LOWDOWN_PARAGRAPH:
+	case LOWDOWN_DEFINITION_TITLE:
 		/*
 		 * Collapse leading white-space if we're already within
 		 * a margin-bearing block statement.
@@ -518,7 +521,7 @@ rndr_buf_startline_prefixes(struct term *term,
 		}
 		break;
 	case LOWDOWN_BLOCKCODE:
-		if (!rndr_buf_style(out, &sinner))
+		if (!rndr_buf_style(term, out, &sinner))
 			return 0;
 		pstyle = 1;
 		if (!HBUF_PUTSL(out, "      "))
@@ -526,7 +529,7 @@ rndr_buf_startline_prefixes(struct term *term,
 		rndr_buf_advance(term, 6);
 		break;
 	case LOWDOWN_ROOT:
-		if (!rndr_buf_style(out, &sinner))
+		if (!rndr_buf_style(term, out, &sinner))
 			return 0;
 		pstyle = 1;
 		for (i = 0; i < term->hmargin; i++)
@@ -535,31 +538,31 @@ rndr_buf_startline_prefixes(struct term *term,
 		break;
 	case LOWDOWN_BLOCKQUOTE:
 		rndr_node_style_apply(&sinner, &sty_bkqt_pfx);
-		if (!rndr_buf_style(out, &sinner))
+		if (!rndr_buf_style(term, out, &sinner))
 			return 0;
 		pstyle = 1;
-		if (!HBUF_PUTSL(out, "  | "))
+		if (!HBUF_PUTSL(out, "    | "))
 			return 0;
-		rndr_buf_advance(term, 4);
+		rndr_buf_advance(term, 6);
 		break;
 	case LOWDOWN_DEFINITION_DATA:
 		rndr_node_style_apply(&sinner, &sty_ddata_pfx);
-		if (!rndr_buf_style(out, &sinner))
+		if (!rndr_buf_style(term, out, &sinner))
 			return 0;
 		pstyle = 1;
-		if (emit == 0 && !HBUF_PUTSL(out, "  : "))
+		if (emit == 0 && !HBUF_PUTSL(out, "    : "))
 			return 0;
-		else if (emit != 0 && !HBUF_PUTSL(out, "    "))
+		else if (emit != 0 && !HBUF_PUTSL(out, "      "))
 			return 0;
-		rndr_buf_advance(term, 4);
+		rndr_buf_advance(term, 6);
 		break;
 	case LOWDOWN_FOOTNOTE_DEF:
 		rndr_node_style_apply(&sinner, &sty_fdef_pfx);
-		if (!rndr_buf_style(out, &sinner))
+		if (!rndr_buf_style(term, out, &sinner))
 			return 0;
 		pstyle = 1;
 		if (emit == 0 && !hbuf_printf
-		    (out, "%2zu. ", n->rndr_footnote_def.num + 1))
+		    (out, "%2zu. ", n->rndr_footnote_def.num))
 			return 0;
 		else if (emit != 0 && !HBUF_PUTSL(out, "    "))
 			return 0;
@@ -570,7 +573,7 @@ rndr_buf_startline_prefixes(struct term *term,
 
 		if (n->rndr_header.level == 0)
 			break;
-		if (!rndr_buf_style(out, &sinner))
+		if (!rndr_buf_style(term, out, &sinner))
 			return 0;
 		pstyle = 1;
 		for (i = 0; i < n->rndr_header.level + 1; i++)
@@ -584,27 +587,40 @@ rndr_buf_startline_prefixes(struct term *term,
 		if (n->parent == NULL ||
 		    n->parent->type == LOWDOWN_DEFINITION_DATA)
 			break;
-		if (n->parent->type == LOWDOWN_LIST &&
-		    (n->parent->rndr_list.flags & HLIST_FL_ORDERED))
-			rndr_node_style_apply(&sinner, &sty_oli_pfx);
-		else
-			rndr_node_style_apply(&sinner, &sty_uli_pfx);
-		if (!rndr_buf_style(out, &sinner))
-			return 0;
-		pstyle = 1;
-		if (n->parent->rndr_list.flags & HLIST_FL_UNORDERED) {
-			if (!hbuf_puts
-			    (out, emit == 0 ?  "  - " : "    "))
+
+		/* Don't print list item prefix after first. */
+
+		if (emit) {
+			if (!HBUF_PUTSL(out, "      "))
 				return 0;
-			rndr_buf_advance(term, 4);
+			rndr_buf_advance(term, 6);
 			break;
 		}
-		if (emit == 0 && !hbuf_printf
-		    (out, "%2zu. ", n->rndr_listitem.num))
+
+		/* List item prefix depends upon type. */
+
+		fl = n->rndr_list.flags;
+		rndr_node_style_apply(&sinner,
+			(fl & HLIST_FL_ORDERED) ?
+			&sty_oli_pfx : &sty_uli_pfx);
+		if (!rndr_buf_style(term, out, &sinner))
 			return 0;
-		else if (emit != 0 && !HBUF_PUTSL(out, "    "))
-			return 0;
-		rndr_buf_advance(term, 4);
+		pstyle = 1;
+
+		if (fl & HLIST_FL_CHECKED) {
+			if (!hbuf_puts(out, "    ☑ "))
+				return 0;
+		} else if (fl & HLIST_FL_UNCHECKED) {
+			if (!hbuf_puts(out, "    ☐ "))
+				return 0;
+		} else if (fl & HLIST_FL_UNORDERED) {
+			if (!hbuf_puts(out, "    · "))
+				return 0;
+		} else {
+			if (!hbuf_printf(out, "%4zu. ", n->rndr_listitem.num))
+				return 0;
+		}
+		rndr_buf_advance(term, 6);
 		break;
 	default:
 		break;
@@ -636,7 +652,7 @@ rndr_buf_startline(struct term *term, struct lowdown_buf *out,
 		return 0;
 	if (osty != NULL)
 		rndr_node_style_apply(&s, osty);
-	return rndr_buf_style(out, &s);
+	return rndr_buf_style(term, out, &s);
 }
 
 /*
@@ -671,7 +687,7 @@ rndr_buf_startwords(struct term *term, struct lowdown_buf *out,
 	rndr_buf_startwords_style(n, &s);
 	if (osty != NULL)
 		rndr_node_style_apply(&s, osty);
-	return rndr_buf_style(out, &s);
+	return rndr_buf_style(term, out, &s);
 }
 
 /*
@@ -1232,7 +1248,7 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 	case LOWDOWN_FOOTNOTE_REF:
 		hbuf_truncate(p->tmp);
 		if (!hbuf_printf(p->tmp, "[%zu]", 
-		    n->rndr_footnote_ref.num + 1))
+		    n->rndr_footnote_ref.num))
 			return 0;
 		rc = rndr_buf(p, ob, n, p->tmp, NULL);
 		break;
@@ -1390,16 +1406,20 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 
 int
 lowdown_term_rndr(struct lowdown_buf *ob,
-	struct lowdown_metaq *mq, void *arg, 
-	const struct lowdown_node *n)
+	void *arg, const struct lowdown_node *n)
 {
-	struct term	*p = arg;
+	struct term		*p = arg;
+	struct lowdown_metaq	 metaq;
+	int			 rc;
 
-	/* Reset ourselves to a sane parse point. */
+	TAILQ_INIT(&metaq);
 
 	p->stackpos = 0;
 
-	return rndr(ob, mq, p, n);
+	rc = rndr(ob, &metaq, p, n);
+
+	lowdown_metaq_free(&metaq);
+	return rc;
 }
 
 void *
